import { Text } from "../../doc/src";
import { EditorSelection } from "./selection";
import { Plugin, StateField } from "./plugin";
import { Transaction } from "./transaction";
declare class Configuration {
    readonly plugins: ReadonlyArray<Plugin>;
    readonly fields: ReadonlyArray<StateField<any>>;
    readonly multipleSelections: boolean;
    readonly tabSize: number;
    readonly lineSeparator: string | null;
    constructor(plugins: ReadonlyArray<Plugin>, fields: ReadonlyArray<StateField<any>>, multipleSelections: boolean, tabSize: number, lineSeparator: string | null);
    static create(config: EditorStateConfig): Configuration;
    updateTabSize(tabSize: number): Configuration;
    updateLineSeparator(lineSep: string | null): Configuration;
}
export interface EditorStateConfig {
    doc?: string | Text;
    selection?: EditorSelection;
    plugins?: ReadonlyArray<Plugin>;
    tabSize?: number;
    lineSeparator?: string | null;
    multipleSelections?: boolean;
}
export declare class EditorState {
    private readonly config;
    readonly doc: Text;
    readonly selection: EditorSelection;
    /** @internal */
    constructor(config: Configuration, doc: Text, selection?: EditorSelection);
    getField<T>(field: StateField<T>): T | undefined;
    readonly plugins: ReadonlyArray<Plugin>;
    getPluginWithField(field: StateField<any>): Plugin;
    /** @internal */
    applyTransaction(tr: Transaction): EditorState;
    readonly transaction: Transaction;
    readonly tabSize: number;
    readonly multipleSelections: boolean;
    joinLines(text: ReadonlyArray<string>): string;
    splitLines(text: string): string[];
    static create(config?: EditorStateConfig): EditorState;
}
export {};
