import { Text } from "../../doc/src";
import { EditorState } from "./state";
import { EditorSelection, SelectionRange } from "./selection";
import { Change, ChangeSet } from "./change";
export declare class MetaSlot<_T> {
    /** @internal */
    name: string;
    constructor(debugName?: string);
    static time: MetaSlot<number>;
    static changeTabSize: MetaSlot<number>;
    static changeLineSeparator: MetaSlot<string | null>;
    static preserveGoalColumn: MetaSlot<boolean>;
    static userEvent: MetaSlot<string>;
    static addToHistory: MetaSlot<boolean>;
}
export declare class Transaction {
    readonly startState: EditorState;
    readonly changes: ChangeSet;
    readonly docs: ReadonlyArray<Text>;
    readonly selection: EditorSelection;
    private readonly meta;
    private readonly flags;
    private constructor();
    static start(state: EditorState, time?: number): Transaction;
    readonly doc: Text;
    setMeta<T>(slot: MetaSlot<T>, value: T): Transaction;
    getMeta<T>(slot: MetaSlot<T>): T | undefined;
    change(change: Change, mirror?: number): Transaction;
    replace(from: number, to: number, text: string | ReadonlyArray<string>): Transaction;
    replaceSelection(text: string | ReadonlyArray<string>): Transaction;
    reduceRanges(f: (transaction: Transaction, range: SelectionRange) => (Transaction | {
        transaction: Transaction;
        range: SelectionRange;
    })): Transaction;
    mapRanges(f: (range: SelectionRange) => SelectionRange): Transaction;
    setSelection(selection: EditorSelection): Transaction;
    readonly selectionSet: boolean;
    readonly docChanged: boolean;
    scrollIntoView(): Transaction;
    readonly scrolledIntoView: boolean;
    apply(): EditorState;
    invertedChanges(): ChangeSet<Change>;
}
